/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.constraint;

import java.util.LinkedList;
import java.util.List;

import edg.config.Config;
import edg.slicing.Phase;

public abstract class Constraint
{
	protected final Config config = Config.getConfig();

	public abstract boolean equals(Object object);
	public abstract String toString();

	protected List<Constraints> wrap(Constraints... constraints)
	{
		final List<Constraints> constraintsList = new LinkedList<Constraints>();

		for (Constraints constraints0 : constraints)
			constraintsList.add(constraints0);

		return constraintsList;
	}

	protected void check(Phase phase1, Phase phase2)
	{
		if (!phase1.isInstanceof(phase2))
			throw new RuntimeException("Constraint situation not contemplated");
	}
	protected <R> void check(R obj1, R obj2)
	{
		if (obj1 == null && obj2 == null)
			return;
		if (obj1 == null || obj2 == null)
			throw new RuntimeException("Constraint situation not contemplated");
		if (obj1 == obj2)
			return;
		if (!obj1.equals(obj2))
			throw new RuntimeException("Constraint situation not contemplated");
	}
}