/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.constraint;

import java.util.HashSet;
import java.util.Set;
import java.util.Stack;

public class Constraints
{
	public static enum Type { Node, Edge }

	protected final Set<NodeConstraint> nodeConstraints = new HashSet<NodeConstraint>();
	protected final Stack<EdgeConstraint> edgeConstraints = new Stack<EdgeConstraint>();

	public Set<NodeConstraint> getNodeConstraints()
	{
		return this.nodeConstraints;
	}
	public void clearNodeConstraints()
	{
		this.nodeConstraints.clear();
	}

	public EdgeConstraint pushEdgeConstraint(EdgeConstraint constraint)
	{
		return this.edgeConstraints.push(constraint);
	}
	public EdgeConstraint popEdgeConstraint()
	{
		return this.edgeConstraints.pop();
	}
	public EdgeConstraint peekEdgeConstraint()
	{
		return this.edgeConstraints.peek();
	}
	public Stack<EdgeConstraint> getEdgeConstraints()
	{
		return this.edgeConstraints;
	}
	public EdgeConstraint getEdgeConstraint(int index)
	{
		return this.edgeConstraints.get(index);
	}

	public boolean isEdgeConstraintsEmpty()
	{
		return this.edgeConstraints.isEmpty();
	}
	public int sizeEdgeConstraints()
	{
		return this.edgeConstraints.size();
	}

	public Object clone()
	{
		final Constraints constraints = new Constraints();

		constraints.nodeConstraints.addAll(this.nodeConstraints);
		constraints.edgeConstraints.addAll(this.edgeConstraints);

		return constraints;
	}
	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof Constraints))
			return false;

		final Constraints constraints = (Constraints) object;

		if (this.nodeConstraints.size() != constraints.nodeConstraints.size())
			return false;
		if (this.edgeConstraints.size() != constraints.edgeConstraints.size())
			return false;
		if (!this.nodeConstraints.equals(constraints.nodeConstraints))
			return false;
		if (!this.edgeConstraints.equals(constraints.edgeConstraints))
			return false;

		return true;
	}
	public int hashCode()
	{
		return this.nodeConstraints.size() + this.edgeConstraints.size();
	}
}