/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.constraint;

public class DataConstructorConstraint extends AccessConstraint
{
	private final String index;

	public DataConstructorConstraint(Operation operation)
	{
		this(operation, null);
	}
	public DataConstructorConstraint(Operation operation, String index)
	{
		super(operation, AccessConstraint.CompositeType.DataConstructor);

		this.index = index;
	}

	public String getIndex()
	{
		return this.index;
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof DataConstructorConstraint))
			return false;

		final DataConstructorConstraint constraint = (DataConstructorConstraint) object;

		if (!super.equals(constraint))
			return false;
		if (this.index == null || constraint.index == null)
			if (this.index != null || constraint.index != null)
				return false;
		if (this.index != null && !this.index.equals(constraint.index))
			return false;
		return true;
	}
	public String toString()
	{
		if (this.index != null)
			return super.toString() + this.index;
		return super.toString();
	}

	public DataConstructorConstraint opposite()
	{
		if (this.operation == Operation.Add)
			return new DataConstructorConstraint(Operation.Remove, this.index);
		return new DataConstructorConstraint(Operation.Add, this.index);
	}
	public boolean cancels(AccessConstraint constraint)
	{
		if (!(constraint instanceof DataConstructorConstraint))
			return super.cancels(constraint);

		final DataConstructorConstraint dataConstructorConstraint = (DataConstructorConstraint) constraint;
		if (dataConstructorConstraint.index != null && dataConstructorConstraint.index.equals("*"))
			return true;
		return super.cancels(constraint);
	}
}