/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.constraint;

import java.util.List;

import edg.graph.Edge;
import edg.slicing.Phase;

public class EmptyConstraint extends EdgeConstraint
{
	private static EmptyConstraint constraint = new EmptyConstraint();
	public static EmptyConstraint getConstraint()
	{
		return EmptyConstraint.constraint;
	}

	private EmptyConstraint()
	{
		
	}

	public boolean equals(Object object)
	{
		return object instanceof EmptyConstraint;
	}
	public String toString()
	{
		return "";
	}

	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, int productionDepth)
	{
		return super.wrap(constraints);
	}
	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, AccessConstraint topConstraint, int productionDepth)
	{
		return super.wrap(constraints);
	}
	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, GrammarConstraint topConstraint, int productionDepth)
	{
		super.check(phase, Phase.SummaryGeneration);

		return super.wrap(constraints);
	}
	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, SeekingConstraint topConstraint, int productionDepth)
	{
		if (topConstraint.operation == SeekingConstraint.Operation.LetThrough)
			super.check(phase, Phase.SummaryGeneration);
		if (topConstraint.operation == SeekingConstraint.Operation.Add || topConstraint.operation == SeekingConstraint.Operation.LetThrough)
			return super.wrap();

		super.check(topConstraint.operation, SeekingConstraint.Operation.Remove);
		super.check(phase, Phase.SummaryGeneration);

		return super.wrap(constraints);
	}
	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, AsteriskConstraint topConstraint, int productionDepth)
	{
		super.check(phase, Phase.SummaryGeneration);

		return super.wrap(constraints);
	}
}