/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.constraint;

public class GlobalVariableConstraint extends SeekingConstraint
{
	private final String variableName;

	public GlobalVariableConstraint(Operation operation, String variableName)
	{
		super(operation, SeekingConstraint.CompositeType.GlobalVariable);

		this.variableName = variableName;
	}

	public String getVariableName()
	{
		return this.variableName;
	}

	public GlobalVariableConstraint opposite()
	{
		if (this.operation == Operation.Add)
			return new GlobalVariableConstraint(Operation.Remove, this.variableName);
		if (this.operation == Operation.Remove)
			return new GlobalVariableConstraint(Operation.Add, this.variableName);
		return new GlobalVariableConstraint(Operation.LetThrough, this.variableName);
	}
	public boolean letThrough(SeekingConstraint constraint)
	{
		if (!(constraint instanceof GlobalVariableConstraint))
			return false;

		final GlobalVariableConstraint gvConstraint = (GlobalVariableConstraint) constraint;

		if (!super.letThrough(gvConstraint))
			return false;
		if (!this.variableName.equals(gvConstraint.variableName))
			return false;
		return true;
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof GlobalVariableConstraint))
			return false;

		final GlobalVariableConstraint constraint = (GlobalVariableConstraint) object;

		if (!super.equals(constraint))
			return false;
		if (!this.variableName.equals(constraint.variableName))
			return false;
		return true;
	}
	public String toString()
	{
		return super.toString() + this.variableName;
	}
}