/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.constraint;

import java.util.LinkedList;
import java.util.List;

import edg.graph.Edge;
import edg.graph.EdgeInfo;
import edg.slicing.Phase;

public class IgnoreEdgeConstraint extends NodeConstraint
{
	private final List<EdgeInfo.Type> types = new LinkedList<EdgeInfo.Type>();

	public IgnoreEdgeConstraint(EdgeInfo.Type type, EdgeInfo.Type... types)
	{
		this.types.add(type);
		for (EdgeInfo.Type type0 : types)
			this.types.add(type0);
	}

	public void resolve(Phase phase, List<Edge> edges)
	{
		edges.removeIf(edge -> this.types.contains(edge.getData().getType()));
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof IgnoreEdgeConstraint))
			return false;

		final IgnoreEdgeConstraint constraint = (IgnoreEdgeConstraint) object;

		if (!this.types.equals(constraint.types))
			return false;
		return true;
	}
	public String toString()
	{
		String toString = "";

		for (EdgeInfo.Type type : this.types)
			toString += type.name() + ",";
		if (!toString.isEmpty())
			toString = toString.substring(0, toString.length() - 1);

		return "I-[" + toString + "]";
	}
}