/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.constraint;

public class ListConstraint extends AccessConstraint
{
	public enum Position { H, T }

	protected final Position position;

	public ListConstraint(Operation operation)
	{
		this(operation, null);
	}
	public ListConstraint(Operation operation, Position position)
	{
		super(operation, AccessConstraint.CompositeType.List);

		this.position = position;
	}

	public Position getPosition()
	{
		return this.position;
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof ListConstraint))
			return false;

		final ListConstraint constraint = (ListConstraint) object;

		if (!super.equals(constraint))
			return false;
		if (this.position != constraint.position)
			return false;
		return true;
	}
	public String toString()
	{
		if (this.position != null)
			return super.toString() + this.position.toString();
		return super.toString();
	}

	public ListConstraint opposite()
	{
		if (this.operation == Operation.Add)
			return new ListConstraint(Operation.Remove, this.position);
		return new ListConstraint(Operation.Add, this.position);
	}
}