/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.constraint;

import java.util.Arrays;
import java.util.List;

import edg.graph.Edge;
import edg.slicing.Phase;

public class PhaseConstraint extends EdgeConstraint
{
	private final List<Phase> allowedPhases;

	public PhaseConstraint(Phase... phases)
	{
		this.allowedPhases = Arrays.asList(phases);
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof PhaseConstraint))
			return false;

		final PhaseConstraint constraint = (PhaseConstraint) object;

		if (this.allowedPhases.equals(constraint.allowedPhases))
			return false;
		return true;
	}
	public String toString()
	{
		if (this.allowedPhases.size() == 1)
			return this.allowedPhases.get(0).name();
		return this.allowedPhases.toString();
	}

	private boolean allows(Phase phase)
	{
		for (Phase allowedPhase : this.allowedPhases)
			if (phase.isInstanceof(allowedPhase))
				return true;
		return false;
	}

	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, int productionDepth)
	{
		if (this.allows(phase))
			return super.wrap(constraints);
		return super.wrap();
	}
	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, AccessConstraint topConstraint, int productionDepth)
	{
		if (this.allows(phase))
			return super.wrap(constraints);
		return super.wrap();
	}
	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, GrammarConstraint topConstraint, int productionDepth)
	{
		super.check(phase, Phase.SummaryGeneration);

		if (this.allows(phase))
			return super.wrap(constraints);
		return super.wrap();
	}
	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, SeekingConstraint topConstraint, int productionDepth)
	{
		if (this.allows(phase))
			return super.wrap(constraints);
		return super.wrap();
	}
	protected List<Constraints> resolve(Phase phase, Edge edge, Constraints constraints, AsteriskConstraint topConstraint, int productionDepth)
	{
		super.check(phase, Phase.SummaryGeneration);

		if (this.allows(phase))
			return super.wrap(constraints);
		return super.wrap();
	}
}