/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.graph;

import edg.constraint.EdgeConstraint;
import edg.constraint.EmptyConstraint;
import edg.constraint.AsteriskConstraint;

public class EdgeInfo
{
	public static enum Type
	{
		ControlFlow,
		Structural, Control,
		Value, Flow,
		Call, Input, Output, Summary, Exception
	}

	private final Type type;
	private final EdgeConstraint constraint;

	public EdgeInfo(Type type)
	{
		this(type, type == Type.Structural || type == Type.Control ? AsteriskConstraint.getConstraint() : EmptyConstraint.getConstraint());
	}
	public EdgeInfo(Type type, EdgeConstraint constraint)
	{
		this.type = type;
		this.constraint = constraint;
	}

	public Type getType()
	{
		return this.type;
	}
	public EdgeConstraint getConstraint()
	{
		return this.constraint;
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof EdgeInfo))
			return false;

		final EdgeInfo edgeInfo = (EdgeInfo) object;

		if (this.type != edgeInfo.type)
			return false;
		if (this.constraint == edgeInfo.constraint)
			return true;
		if (this.constraint == null || edgeInfo.constraint == null)
			return false;
		if (!this.constraint.equals(edgeInfo.constraint))
			return false;
		return true;
	}
}