/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.graph;

import edg.LDASTNodeInfo;

public class NodeInfo
{
	public static enum Type
	{
		// Module
		Module,

		// Routine
		Routine, Clause, Parameters,

		// Expressions
		List, DataConstructor, DataConstructorAccess,
		Block, Operation, Equality,
		If, Condition,
		Switch, Selector, Cases, Case, DefaultCase, Selectable,
		Call, Callee, Scope, Name, Arguments,
		ListComprehension, Restrictions, Generator, Filter, Value,
		Loop,

		// Others
		Body, Guard,
		Expression, Result,
		Variable, Literal,
		Return, Break, Continue,
		Root
	}

	private final int id;
	private final Type type;
	private final String name;
	private final LDASTNodeInfo ldASTNodeInfo;

	public NodeInfo(int id, Type type, String name, LDASTNodeInfo ldASTNodeInfo)
	{
		this.id = id;
		this.type = type;
		this.name = name;
		this.ldASTNodeInfo = ldASTNodeInfo;
	}

	public int getId()
	{
		return this.id;
	}
	public Type getType()
	{
		return this.type;
	}
	public String getName()
	{
		return this.name;
	}
	public LDASTNodeInfo getInfo()
	{
		return this.ldASTNodeInfo;
	}

	public boolean isFictitious()
	{
		switch (this.type)
		{
			case Root:
			case Parameters:
			case Guard:
			case Body:
			case Selector:
			case Cases:
			case Selectable:
			case Callee:
			case Arguments:
			case Expression:
			case Result:
			case Restrictions:
			case Value:
				return true;
			default:
				return false;
		}
	}
}