/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.slicing;

public class SlicingCriterion
{
	protected final String archive;
	protected final int line;
	protected final String name;
	protected final int occurrence;

	public SlicingCriterion(String archive, int line, String name)
	{
		this(archive, line, name, 1);
	}
	public SlicingCriterion(String archive, int line, String name, int occurrence)
	{
		this.archive = archive;
		this.line = line;
		this.name = name;
		this.occurrence = occurrence;
	}

	public String getArchive()
	{
		return this.archive;
	}
	public int getLine()
	{
		return this.line;
	}
	public String getName()
	{
		return this.name;
	}
	public int getOccurrence()
	{
		return this.occurrence;
	}

	public boolean equals(Object obj)
	{
		if (obj == this)
			return true;
		if (!(obj instanceof SlicingCriterion))
			return false;

		final SlicingCriterion sc = (SlicingCriterion) obj;

		if (!this.archive.equals(sc.archive))
			return false;
		if (this.line != sc.line)
			return false;
		if (!this.name.equals(sc.name))
			return false;
		if (this.occurrence != sc.occurrence)
			return false;
		return true;
	}
	public String toString()
	{
		return "[" + this.archive + ", " + this.line + ", " + this.name + ", " + this.occurrence + "]";
	}
}