/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.slicing;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import edg.graph.Edge;
import edg.graph.EdgeInfo;
import edg.graph.Node;

public class StandardAlgorithm implements SlicingAlgorithm
{
	public List<Node> slice(Node node)
	{
		final List<Node> slice = new LinkedList<Node>();
		if (node == null)
			return slice;

		slice.add(node);
		this.traverse(slice, EdgeInfo.Type.Output);
		this.traverse(slice, EdgeInfo.Type.Input);

		return slice;
	}
	private void traverse(List<Node> slice, EdgeInfo.Type... ignoreEdgeTypes)
	{
		final List<Node> pendingNodes = new LinkedList<Node>(slice);
		final List<EdgeInfo.Type> listIgnoreEdgeTypes = Arrays.asList(ignoreEdgeTypes);

		while (!pendingNodes.isEmpty())
		{
			final Node pendingNode = pendingNodes.remove(0);
			final List<Edge> incomingEdges = pendingNode.getIncomingEdges();

			incomingEdges.removeIf(edge -> edge.getData().getType() == EdgeInfo.Type.ControlFlow);
			incomingEdges.removeIf(edge -> listIgnoreEdgeTypes.contains(edge.getData().getType()));
			for (Edge incomingEdge : incomingEdges)
			{
				final Node nodeFrom = incomingEdge.getFrom();
				if (slice.contains(nodeFrom))
					continue;

				pendingNodes.add(nodeFrom);
				slice.add(nodeFrom);
			}
		}
	}
}